! wtellask:_it.h  Libreria conversazionale su sistema Ask/Tell
!                 Paolo Lucchesi, 2004
!
! ------------------------------------------------------------------------------
!
! Questo file e' software libero, puo' essere liberamente usato, copiato,
! distribuito e modificato. Qualsiasi versione, modificata o meno, di questo
! file distribuita sotto forma di codice sorgente deve contenere riferimento
! all'autore originale e deve essere liberamente usabile, copiabile,
! distribuibile e modificabile.
! Non e' posta alcuna condizione alla distribuzione del file come codice
! compilato.
! Questo file e' distribuito senza alcuna garanzia. L'autore non puo'
! essere ritenuto responsabile in alcun modo di danni provocati
! direttamente o indirettamente da questo file.
!
! ------------------------------------------------------------------------------
!
! Richiesta la libreria 6/11 o superiore, e Infit versione 2.5 o superiore
!
! Come usare questo file:
! Mettere nel vostro file la direttiva
!     include "wtellask_it";
! due volte: dopo aver incluso verblib.h e dopo aver incluso grammar.h
!
! Questa libreria serve ad aiutare gli autori a definire personaggi non
! giocatori che possano rispondere a comandi come 'chiedi al personaggio
! del argomento' e 'parla al personaggio dell'argomento'. Attualmente la
! libreria non fa differenza tra i due comandi.
!
! Ogni personaggio non giocatore deve appartenere alla classe NPC (che ha gia'
! settato l'attributo animate) e la sua proprieta' life non deve contenere i
! casi relativi alle azioni Tell e Ask (e default, come vedremo avanti). Le
! altre azioni vengono gestite normalmente.
! L'oggetto di classe NPC deve avere una proprieta' conversor puntata ad
! un oggetto che e' il controllore della conversazione. Tutti i possibili
! argomenti di conversazione devono essere figli di questo controllore, cosi'
! potete aggiungerli e sottrarli in qualsiasi momento (anche se di rado vi
! capitera' di sfruttare questa possibilita').
! L'oggetto conversor contiene nella proprieta' description il frammento
! di dialogo che viene usato in risposta al comando 'parla con npc' (senza 
! specificare argomenti). Questo puo' essere una stringa o una routine (che
! puo' reagire diversamente a seconda dello stato del gioco).
! La proprieta' description puo' anche essere una lista di stringhe o
! routine, che verranno usate sequenzialmente per ogni successivo comando
! 'parla con npc'; l'ultimo elemento della lista viene ripetuto per ogni
! volta successiva. La proprieta' cnumber dell'NPC tiene conto di
! quante volte e' stato usato il comando 'parla con npc'.
! Inoltre l'oggetto conversor contiene nella proprieta' cant_go il frammento 
! usato quando non viene trovata nessuna  corrispondenza per l'argomento di 
! conversazione richesto (quando il p.n.g. non sa dire niente su un dato 
! argomento, ad esempio "Mi spiace, non ne so niente...").
! Infine se l'oggetto conversor ha una proprieta' before, questa viene eseguita
! prima di ogni altra cosa (ovviamente in risposta ad un'azione Tell o Ask) e,
! se essa restituisce true, non ci sara' altra risposta al comando; alla
! proprieta' before viene passato un singolo argomento che vale true se
! il comando era 'parla con npc' - senza argomenti di conversazione, e false 
! altrimenti. (nota: la proprieta' before non incrementa cnumber)
!
! Tutti gli argomenti devono appartenere alla classe Topic.
! La proprieta' name di ciascun argomento contiene la lista di nomi con cui
! il giocatore puo' riferirsi ad un certo argomento.
! La proprieta' description invece contiene il frammento di dialogo vero e 
! proprio, e puo' essere una stringa o una routine; in quest'ultimo caso
! la routine puo' fare qualsiasi cosa, e reagire in modo diverso se cambiano
! le condizioni in cui viene chiamata.
! La proprieta' description puo' anche essere una lista di stringhe o 
! routine, che verranno chiamate sequenzialmente ogni volta che l'argomento
! viene trattato. L'ultimo elemento della lista viene ripetuto per ogni
! volta successiva (ma vedi la proprieta' metatopic). La proprieta' cnumber
! dell'argomento tiene conto delle volte che l'argomento e' stato trattato.
! Opzionalmente ogni argomento puo' avere la proprieta' after, che viene
! eseguita dopo la visualizzazione del dialogo.
! Inoltre ogni argomento puo' avere anche la proprieta' before, che viene
! eseguita prima della visualizzazione del dialogo. Se la proprieta' before
! restituisce true, il dialogo non viene mostrato - le proprieta' description
! e after non vengono valutate (nota: la proprieta' before non incrementa
! cnumber).
! Ogni oggetto di classe Topic puo' avere settato l'attributo locked. In
! questo caso la libreria assume che il protagonista non abbia mai sentito
! parlare di tale argomento e non gli permette di parlare. La proprieta'
! before viene eseguita anche se l'attributo locked e' settato (e puo'
! modificare l'attributo locked).
! Le costanti LOCKEDANSWER e LOCKEDANSWERPRE possono venir definite come
! risposte in caso di argomento 'locked'; normalmente viene stampata 
! LOCKEDANSWER, ma se la variabile pretalked (che viene posta a zero 
! all'inizio di ogni frammento di conversazione) e'  stata settata - 
! probabilmente in una before - ad un valore diverso da zero, viene 
! invece stampata LOCKEDANSWERPRE.
! Se necessario, si puo' visualizzare un argomento (esattamente come se fosse
! stato richiamato con 'parla a npc di argomento') invocando la proprieta'
! argomento.act().
!
! ATTENZIONE: non si puo' piu' usare il caso default: della proprieta' life;
! al suo posto si deve usare la proprieta' lifedefault.
!
! Ogni oggetto di classe Topic puo' avere anche la proprieta' metatopic,
! puntata ad un altro oggetto di classe Topic; quest'ultimo oggetto dovrebbe
! contenere dati generali relativi ad un certo argomento, indipendentemente
! dalla persona con cui si sta parlando. In pratica se viene definito un
! metatopic, si hanno tre effetti:
! - la lista dei nomi validi e' presa dalla proprieta' name sia dell'argomento
!   vero e proprio, sia dal suo metatopic
! - anche se il metatopic ha settato l'attributo locked, l'argomento di dialogo
!   e' bloccato
! - gli elementi della proprieta' description del metatopic (se esiste) vengono
!   aggiunti alla lista della proprieta' description dell'argomento principale
!   (in particolare, se quest'ultimo non ha una proprieta' description, viene 
!   presa quella del metatopic - se esiste)
!
! Inoltre la libreria aggiunge i comandi 'parla dell'argomento' (abbreviato in
! 't argomento') e 'chiedi dell'argomento' che si rivolgono al personaggio (se 
! esiste) con cui il protagonista stava gia' parlando.
! Normalmente tale personaggio e' l'ultimo a cui il giocatore ha rivolto la
! parola (purche' sia sempre rimasto in scope), ed e' indicato dalla variabile
! talkingwith. 
! L'autore puo' far iniziare una conversazione con un certo png chiamando 
! png.talkto(flag) o talker(png, flag), dove png e' il personaggio con cui
! iniziare una nuova conversazione e flag e' una variabile true/false che 
! indica se la conversazione deve essere iniziata anche se il protagonista 
! sta gia' parlando con qualcun'altro.
! La variabile silenceturns contiene il numero di turni trascorsi
! dall'ultimo comando di dialogo.
!
! Esempio
!
! Topic metapalazzo
!   with name 'palazzo' 'biblioteca' 'edificio';
!   
! Topic metalibro
!   with name 'libro' 'nero',
!   has locked;
!   
! NPC -> bibliotecaria "bibliotecaria"
!   with name 'ragazza' 'giovane' 'donna' 'bibliotecaria',
!     description "Ancora giovane e decisamente attraente.",
!     conversor ragazzatalk, ! <--- controllore di conversazione
!     Life [;
!       Answer: "Non credi che la ragazza possa esserti d'aiuto.";
!       Show, Give: 
!             print_ret "La ragazza osserva ", (the)noun, ", ma non pare troppo colpita.";      
!       ThrowAt, Attack: "Ti ha forse fatto qualcosa di male?";
!     ],
!     lifedefault [; "La ragazza ti osserva con curiosit@`a."; ], ! <--- al posto di Default:
!   has female;
!
! Object ragazzatalk 
!   with description "~Ciao, come va?~^
!                    ~Non male... E a te?~",   ! <--- risposta a 'parla alla ragazza'
!        cant_go "~Mi spiace. Non ne so niente...~";   ! <--- se non vengono trovati argomenti
!
! Topic -> 
!    with metatopic metapalazzo,
!    description [;
!      give metalibro ~unlocked; ! <- Ora possiamo chiedere del libro
!      "~Cosa mi sai dire di questa biblioteca?~^
!       ~E' stata costruita nel 1666. Contiene diversi testi altrimenti introvabili,
!        come quel libro nero.~";
!     ];
!             
! Topic -> 
!    with name 'volume' 'tomo',
!    metatopic metalibro,
!    description [;
!        if (self hasnt general) { ! <--- Dialogo che cambia
!            give self general;
!            "~Mi sai dire qualcosa di quel libro dalla copertina nera?~^
!             ~Quello @`e il libro di Eibon. E' molto pericoloso.~";
!        }
!        "~A proposito di quel libro nero...~^
!         ~Come ho gia' detto, meglio lasciarlo stare.~";
!    ];
!                  
! Topic ->
!    with name 'quadro' 'dipinto' 'ritratto',
!    ! Questo e' un esempio di lista di frasi
!    description "~Chi @`e l'uomo rappresentato in quel ritratto?~^
!                 ~Il conte Parnias, era proprietario della biblioteca.~"
!                "~Sa dirmi chi ha dipinto questo quadro?~^
!                 ~Un certo Tommaso Malvasi, non ne so altro."
!                "Non hai altro da chiedere sul quadro.";
!
! ------------------------------------------------------------------------------

! ----
! Prima inclusione

#Ifndef LIBRARY_GRAMMAR;

Default LOCKEDANSWER "Non conosci niente di simile.";
Default LOCKEDANSWERPRE "...^Non conosci niente di simile.";

Global talkingwith=NULL;
Global silenceturns=0;
Global wtopicword=0;
Global pretalked;

Property cnumber;
Property act;
Property talkto;
Property metatopic;

Class NPC
  with 
    number 0, cnumber 0,
	life [wd i x mtx j;
      Tell,Ask:
	    pretalked = 0;
        if (~~(self provides conversor)) rfalse;
        talkingwith = self; silenceturns=0;
        if ((self.conversor) provides before) {   
            !print consult_words;
            x = (consult_words==0 && action==##Tell);
            if ((self.conversor).before(x)) rtrue;
        }
        if (consult_words==0 && action==##Tell) {
			if ((self.conversor) provides description) {
			  j = (self.conversor).#description/WORDSIZE;
			  if (j <= 0) rfalse;
			  self.cnumber++; 
			  if (self.cnumber < 1) self.cnumber=1;
			  if (self.cnumber > j) self.cnumber = j;
              x=(self.conversor).&description-->(self.cnumber-1);
              if (metaclass(x) == Routine) x(); else print (string) x, "^";
			  rtrue;
            }
            rfalse;
        }
        wn=consult_from;
        for (i=0: i<consult_words: i++) {     
            wd=NextWord(); wtopicword=wd;
            objectloop (x in self.conversor) {
                if (x ofclass Topic) {
                    mtx = NULL;
                    if (x provides metatopic) mtx = x.metatopic;
                    if (~~(mtx ofclass Topic)) mtx=NULL;
                    if (mtx ~= NULL or 0) {
                        for (j=0: j<((mtx.#name) / WORDSIZE): j++) {
                            if ( ( wd ) == ( (mtx.&name-->j) ) ) {
                                dowdialog(x, mtx);
                                rtrue;
                            }
                        }
                    }
                    for (j=0: j<((x.#name) / WORDSIZE): j++) {
                        !print i, " ", j, " : ", wd, " ", x&name-->j;
                        if ( ( wd ) == ( (x.&name-->j) ) ) {
                            dowdialog(x, mtx);
                            rtrue;
                        }
                    }
                }
            }
        }
        if ((self.conversor) provides cant_go) {
            PrintOrRun((self.conversor), cant_go, 0);
            rtrue;
        }
	  Greet: 
          ! <<Tell self>>;
		  pretalked = 0;
          if (~~(self provides conversor)) rfalse;
          talkingwith = self; silenceturns=0;   
		  if ((self.conversor) provides before)               
              if ((self.conversor).before(true)) rtrue;
          if ((self.conversor) provides description) {
			  j = (self.conversor).#description/WORDSIZE;
			  if (j <= 0) rfalse;
			  if (self.cnumber < 1) self.cnumber=1;
			  if (self.cnumber > j) self.cnumber = j;
              x=(self.conversor).&description-->0;
              if (metaclass(x) == Routine) x(); else print (string) x, "^";
			  rtrue;          
          }
          rfalse;
      default:
          if (self provides lifedefault) {
              PrintOrRun(self, lifedefault, 0);
              rtrue;
          }
  ],
  talkto [force;
      if ((talkingwith == NULL || force) && (self provides conversor)) 
          { talkingwith = self; silenceturns=0; }
  ],
  has animate;
  
Class Topic
  with number 0,
       cnumber 0,
       act [x;
           if (self provides metatopic) x = self.metatopic; else x = 0;
           dowdialog(self, x);
       ];

Object wtellaskfakeobject
    with metatopic, lifedefault,
      __warningsquasher [;  talker(self,false); ];

[ dowdialog tp mtp rval nt nmt x;
    if (tp provides before) if (tp.before()) rtrue;
	x = 0;
    if (tp has locked) x = 1;
    if (mtp ~= NULL or 0) if (mtp has locked) x = 1;
	if (x) {
	    if (pretalked) print_ret (string)LOCKEDANSWERPRE;
	    print_ret (string)LOCKEDANSWER;
	}
    rval=false;
    nt = 0; nmt = 0;
	if (tp provides description) nt = tp.#description/WORDSIZE;
	if (mtp ~= NULL or 0) if (mtp provides description) nmt = mtp.#description/WORDSIZE;
    if ((nt+nmt) > 0) {
	  tp.cnumber++;
	  ! print nt, " + ", nmt, " ( ", tp.cnumber, " )^";
	  if (tp.cnumber < 1) tp.cnumber=1;
	  if (tp.cnumber > (nt+nmt)) tp.cnumber = (nt+nmt);
	  if (tp.cnumber > nt) x=mtp.&description-->(tp.cnumber-nt-1);
	   else                x=tp.&description-->(tp.cnumber-1);
      if (metaclass(x) == Routine) rval=x(); else { print (string) x, "^"; rval = true; }
    }
    if (tp provides after)
        rval=tp.after();
    rtrue;
];

Object "(wtellask)" LibraryExtensions
  with ext_initialise [;
    TalkInit();
    rtrue;
  ];

[ TalkInit; ! Left for legacy;
    talkingwith = NULL;
    StartDaemon(talkdaemon);
];

Object talkdaemon
  with daemon [;
    if (talkingwith ~= NULL)
        if (~~TestScope(talkingwith)) talkingwith=NULL;   
    if (talkingwith) silenceturns++;      
];

[ talker id force;
      if ((talkingwith == NULL || force || id == NULL) && (id provides conversor)) 
          { talkingwith = id; silenceturns=0; }
];

! Routine per topic
[KeepTalkSub;
  if (talkingwith==NULL) "Non stai parlando con nessuno, adesso.";
  <<Tell talkingwith noun>>;
];

[KeepAskSub;
  if (talkingwith==NULL) "Non stai parlando con nessuno, adesso.";
  <<Ask talkingwith noun>>;
];

! Greet
[GreetSub;
  if (RunLife(noun,##Greet)~=0) rfalse;
  L__M(##Greet,1,noun);
]; 

#Endif;


! ----
! Seconda inclusione

#Ifdef LIBRARY_GRAMMAR;

Extend 'parla'
                * 'circa'/'su'/'sul'/
                  'sullo'/'sull^'/'sulla'/'sugli'/'sui'/
                  'sulle'/'di'/'dello'/'della'/'dell^'/'dei'/
                  'degli'/'delle' topic
                                                 -> KeepTalk;

Extend 'chiedi' first
                * 'circa'/'su'/'sul'/'sui'/
                  'sullo'/'sull^'/'sulla'/'sugli'/
                  'sulle'/'di'/'dello'/'della'/'dell^'/'dei'/
                  'degli'/'delle' topic
                                                 -> KeepAsk;
                                                 
Verb 'topic' 'argomento' 't//'
                * topic                          -> KeepTalk;
				
Extend 'saluta' replace
				*                                -> WaveHands
                * creature                       -> Greet;

#Endif;
